#include <cstring>
#include <cstdio>
#include "RenderChimp.h"
#include "Resource.h"
#include "ResourceList.h"
#include "IndexArray.h"

/*---------------------------------------------------------------------------*/

IndexArray::IndexArray(
		const char				*nidentifier,

		u16						*new_array,
		const u32				n,

		IndexArrayLayout_t		layout,

		u32						purge_level
	) : Resource(nidentifier, purge_level, RESOURCE_INDEX_ARRAY)
{
	if (strlen(nidentifier) == 0)
		REPORT(ERR_ARG);

	n_elements = n;

	indexArray = (u16 *) malloc(sizeof(u16) * n);
	memcpy(indexArray, new_array, sizeof(u16) * n);

	size = sizeof(u16) * n;

	primitiveLayout = layout;

	switch (layout) {
		default:
		case TRIANGLES:
			glLayout = GL_TRIANGLES;
			break;
		case TRIANGLE_STRIP:
			glLayout = GL_TRIANGLE_STRIP;
			break;
		case TRIANGLE_FAN:
			glLayout = GL_TRIANGLE_FAN;
			break;
	}
}

/*---------------------------------------------------------------------------*/

IndexArray::~IndexArray()
{
	if (indexArray)
		free(indexArray);
}

/*---------------------------------------------------------------------------*/

Resource *IndexArray::clone(
		const char		*clone_name,

		u32				purge_level
	)
{
	IndexArray *new_res = new IndexArray(identifier, indexArray,
		n_elements, primitiveLayout, purge_level);

	return new_res;
}

/*---------------------------------------------------------------------------*/

u16 *IndexArray::getArray()
{
	return indexArray;
}

/*---------------------------------------------------------------------------*/

u32 IndexArray::getLength()
{
	return n_elements;
}

/*---------------------------------------------------------------------------*/

u32 IndexArray::getPrimitiveLayout()
{
	return primitiveLayout;
}

/*---------------------------------------------------------------------------*/

u32 IndexArray::getGLPrimitiveLayout()
{
	return glLayout;
}

/*---------------------------------------------------------------------------*/

void IndexArray::setIndexArray(
		IndexArray				**target_index_array,
		IndexArray				*new_index_array,

		bool					instantiate_resource
	)
{
	if (*target_index_array)
		sceneGraph.dereferenceResource(*target_index_array);

	if (instantiate_resource) {
		*target_index_array = (IndexArray *) new_index_array->instantiate();
	} else {
		*target_index_array = new_index_array;
	}
}

/*---------------------------------------------------------------------------*/

#ifdef DEBUG

void IndexArray::print(
		FILE		*f
	)
{

#ifdef LOG_TO_FILE
	#define prnt0(a)		fprintf(f, a)
	#define prnt1(a, b)		fprintf(f, a, b)
#else /* LOG_TO_FILE */
	#define prnt0(a)		printf(a)
	#define prnt1(a, b)		printf(a, b)
#endif /* LOG_TO_FILE */

	prnt1("    [ N ELEMS  ][ %u ]\n", n_elements);
	prnt1("    [ LAYOUT   ][ %u ]\n", (u32) primitiveLayout);

}

#endif /* DEBUG */

/*---------------------------------------------------------------------------*/

