/*
 * Index array.
 * Inherits from Resource.
 *
 * This is an array of indices that is used in conjunction with a vertex
 * array. These indices are grouped three by three to specify which vertices
 * that comprise the triangles.
 */

#ifndef __RC_INDEXARRAY_H__
#define __RC_INDEXARRAY_H__

/*--------------------------------------------------[ Material settings ]----*/

typedef enum {

	TRIANGLES = 0,
	TRIANGLE_STRIP,
	TRIANGLE_FAN,

	INDEX_ARRAY_LAST

} IndexArrayLayout_t;

/*---------------------------------------------------------------------------*/

class IndexArray : public Resource {
	friend class SceneGraph;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		IndexArray(
				const char				*nidentifier,

				u16						*new_array,
				const u32				n,			/* The number of indices in
													 * the index array. (Not
													 * the number of triangles
													 * you expect to be
													 * created.)
													 */

				IndexArrayLayout_t		layout,

				u32						purge_level
			);

		virtual ~IndexArray();

	public:
		Resource *clone(
				const char				*clone_name,

				u32						purge_level
			);

		u16	*getArray();

		u32 getLength();

		u32 getPrimitiveLayout();

		u32 getGLPrimitiveLayout();

		/* Static helper function */
		static void setIndexArray(
			IndexArray				**target_index_array,
			IndexArray				*new_index_array,

			bool					instantiate_resource
		);

#ifdef DEBUG
	protected:
		void print(
				FILE	*f
			);
#endif /* DEBUG */

/*------------------------------------------------------------[ Members ]----*/

	private:
		u16					*indexArray;
		u32					n_elements;

		IndexArrayLayout_t	primitiveLayout;
		u32					glLayout;

};

#endif /* __RC_INDEXARRAY_H__ */

