#include "RenderChimp.h"
#include "SceneGraph.h"
#include "Material.h"
#include "MaterialReflective.h"

/* Warnings */
static const Result_t WRN_NO_TEX = {
	RESULT_WARNING, "The material has no attached texture. Weird shading may follow"};

static const Result_t WRN_NO_LIGHT = {
	RESULT_WARNING, "There is no light source specified for Geometry node"};

/*---------------------------------------------------------------------------*/

/* Shader prefix */
static char *prefix = \
	"\n"
;

/* Shaders */
static char *vshader = "../shaders/MaterialReflective.vs";
static char *fshader = "../shaders/MaterialReflective.fs";

/*---------------------------------------------------------------------------*/

MaterialReflective::MaterialReflective(
		const char		*nidentifier,
		u32				purge_level
	) : Material(nidentifier, purge_level, MATERIAL_REFLECTIVE)
{
	shaderProgram = sceneGraph.createShaderProgram(nidentifier, prefix,
		vshader, fshader, purge_level);

	cubeMap = 0;
}

/*---------------------------------------------------------------------------*/

MaterialReflective::~MaterialReflective()
{
	ensure_heap();

	if (cubeMap)
		sceneGraph.dereferenceResource(cubeMap);

	ensure_heap();
}

/*---------------------------------------------------------------------------*/

void MaterialReflective::prepareMaterial(
		FrameInfo_t			*di,
		Geometry			*geometry
	)
{
	if (cubeMap) {
		shaderProgram->setCubeMapSampler("EnvSampler", cubeMap, 0);
	} else {
		REPORT(WRN_NO_TEX);
	}

	GL_ASSERT();
}

/*---------------------------------------------------------------------------*/

void MaterialReflective::setCubeMap(
		CubeMap			*cube_map,

		bool			instantiate_resource
	)
{
	CubeMap::setCubeMap(&cubeMap, cube_map, instantiate_resource);
}

/*---------------------------------------------------------------------------*/

CubeMap *MaterialReflective::getCubeMap()
{
	return cubeMap;
}

/*---------------------------------------------------------------------------*/

Resource *MaterialReflective::clone(
		const char		*clone_name,

		u32				purge_level
	)
{
	MaterialReflective *new_res = (MaterialReflective *) sceneGraph.createMaterial(materialType,
		clone_name, purge_level);

	new_res->setCubeMap(cubeMap, true);

	return new_res;
}

/*---------------------------------------------------------------------------*/

