#include "RenderChimp.h"
#include "SceneGraph.h"
#include "Material.h"
#include "MaterialTextured.h"

/* Warnings */
static const Result_t WRN_NO_TEX = {
	RESULT_WARNING, "The material has no attached texture. Weird shading may follow"};

static const Result_t WRN_NO_LIGHT = {
	RESULT_WARNING, "There is no light source specified for Geometry node"};

/*---------------------------------------------------------------------------*/

/* Shader prefix */
static char *prefix = \
	"\n"
;

/* Shaders */
static char *vshader = "../shaders/MaterialTextured.vs";
static char *fshader = "../shaders/MaterialTextured.fs";

/*---------------------------------------------------------------------------*/

MaterialTextured::MaterialTextured(
		const char		*nidentifier,
		u32				purge_level
	) : Material(nidentifier, purge_level, MATERIAL_TEXTURED)
{
	shaderProgram = sceneGraph.createShaderProgram(nidentifier, prefix,
		vshader, fshader, purge_level);

	texture = 0;

	Texture::setTexture(&texture, "../textures/notex.png", NO_PURGE);
}

/*---------------------------------------------------------------------------*/

MaterialTextured::~MaterialTextured()
{
	ensure_heap();

	if (texture)
		sceneGraph.dereferenceResource(texture);

	ensure_heap();

}

/*---------------------------------------------------------------------------*/

void MaterialTextured::prepareMaterial(
		FrameInfo_t			*di,
		Geometry			*geometry
	)
{
	if (texture) {
		shaderProgram->setTextureSampler("DiffuseSampler", texture, 0);
	} else {
		REPORT(WRN_NO_TEX);
	}

	GL_ASSERT();
}

/*---------------------------------------------------------------------------*/

void MaterialTextured::setTexture(
		Texture			*tex,

		bool			instantiate_resource
	)
{
	Texture::setTexture(&texture, tex, instantiate_resource);
}

/*---------------------------------------------------------------------------*/

void MaterialTextured::setTexture(
		const char		*filename,

		u32				purge_level
	)
{
	Texture::setTexture(&texture, filename, purge_level);
}

/*---------------------------------------------------------------------------*/

Texture *MaterialTextured::getTexture()
{
	return texture;
}

/*---------------------------------------------------------------------------*/

Resource *MaterialTextured::clone(
		const char		*clone_name,

		u32				purge_level
	)
{
	MaterialTextured *new_res = (MaterialTextured *) sceneGraph.createMaterial(materialType,
		clone_name, purge_level);

	new_res->setTexture(texture, true);

	return new_res;
}

/*---------------------------------------------------------------------------*/

