#include "RenderChimp.h"
#include "SceneGraph.h"
#include "MatrixTransformation.h"

/*---------------------------------------------------------------------------*/

MatrixTransformation::MatrixTransformation()
{
	u32 i;

	for (i = 0; i <	N_TRANSFORMATION_MATRICES; i++)
		transformationMatrices[i] = 0;

}

/*---------------------------------------------------------------------------*/

MatrixTransformation::~MatrixTransformation()
{
	
}

/*---------------------------------------------------------------------------*/

void MatrixTransformation::prepareFrame(
		mat4f				*nview,
		mat4f				*nprojection,
		mat4f				*nview_projection,
		ProjectionType_t	projection_type
	)
{
	u32 i;

	for (i = 0; i < N_TRANSFORMATION_MATRICES; i++)
		transformationMatrices[i] = 0;

	view = nview;
	projection = nprojection;
	viewProjection = nview_projection;

	projectionType = projection_type;

	nextFreeNode = 0;
	nextFreeFrame = 0;

	viewPosition = vec3f(nview->T0, nview->T1, nview->T2);
}

/*---------------------------------------------------------------------------*/

void MatrixTransformation::finishNode()
{
	u32 i;

	/* Reset matrices that has to be computed per node */
	for (i = MAT_WORLD+1; i < N_TRANSFORMATION_MATRICES; i++)
		transformationMatrices[i] = 0;

	nextFreeNode = 0;
}

/*---------------------------------------------------------------------------*/

mat4f *MatrixTransformation::assembleMatrix(
		const u32			index
	)
{
	u32 b = index & (~0x3);
	u32 m = index & 0x3;

	mat4f *matList;
	u32 *nextFree;

	/* TODO: Add optimization that considers ProjectionType_t */

	/* Check whether it's necessary to update the matrix at all */
	if (transformationMatrices[index]) {
		/* No need to update matrix - it already exists */
		return transformationMatrices[index];
	}

	/* If we get this far we have to update our matrix */
	switch (b) {

		case MAT_WORLD_VIEW:

			if (!transformationMatrices[b]) {

				transformationMatrices[index] = &matListNode[nextFreeNode++];
				*transformationMatrices[index] = 
					(*transformationMatrices[MAT_VIEW]) *
					(*transformationMatrices[MAT_WORLD]);

			}

			matList = matListNode;
			nextFree = &nextFreeNode;

		break;

		case MAT_WORLD_VIEW_PROJ:

			if (!transformationMatrices[b]) {

				transformationMatrices[index] = &matListNode[nextFreeNode++];
				*transformationMatrices[index] = 
					(*transformationMatrices[MAT_PROJ]) *
					(*transformationMatrices[MAT_VIEW]) *
					(*transformationMatrices[MAT_WORLD]);

			}

			matList = matListNode;
			nextFree = &nextFreeNode;

		break;

		case MAT_WORLD:
			
			matList = matListNode;
			nextFree = &nextFreeNode;

		break;

		default:

			matList = matListFrame;
			nextFree = &nextFreeFrame;

		break;

	}

	switch (m) {
		default:
		case MAT_BASE:
			
		break;

		case MAT_INV:
			transformationMatrices[index] = &matList[(*nextFree)++];
			*transformationMatrices[index] = *transformationMatrices[b];
			transformationMatrices[index]->inv();
		break;

		case MAT_T:
			transformationMatrices[index] = &matList[(*nextFree)++];
			*transformationMatrices[index] = *transformationMatrices[b];
			transformationMatrices[index]->transpose();
		break;

		case MAT_INV_T:
			transformationMatrices[index] = &matList[(*nextFree)++];

			if (!transformationMatrices[b + MAT_INV]) {
				transformationMatrices[b + MAT_INV] = &matList[(*nextFree)++];
				*transformationMatrices[b + MAT_INV] = *transformationMatrices[b];
				transformationMatrices[b + MAT_INV]->inv();
			}

			*transformationMatrices[index] = *transformationMatrices[b + MAT_INV];
			transformationMatrices[index]->transpose();

		break;

	}

	return transformationMatrices[index];
}

/*---------------------------------------------------------------------------*/

void MatrixTransformation::setTransformationMatrices(
		ShaderProgram			*shaderProgram
	)
{
	u32 m;
	u32 i;

	m = 0;

	/* Look for transformation matrices. */
	for (i = 0; i < N_TRANSFORMATION_MATRICES; i++) {
		GLint loc = shaderProgram->getUniformLocation(transformationMatrixNames[i]);

		if (loc >= 0) {
			/* The transformation keyword exists */

			mat4f *mat = assembleMatrix(i);

			glUniformMatrix4fv(loc, 1, GL_FALSE, mat->s);
		}
	}

	GL_ASSERT();
}

/*---------------------------------------------------------------------------*/

