#include "RenderChimp.h"
#include "Result.h"

FILE *logfile;
char *last_report = 0;
u32 report_occured = 0;

/*---------------------------------------------------------------------------*/

void ResultInit()
{

#ifdef LOG_TO_FILE

	logfile = fopen("log.txt", "w");

#endif /* LOG_TO_FILE */

}

/*---------------------------------------------------------------------------*/

void ResultRelease()
{
	if (last_report)
		free(last_report);

	fclose(logfile);
}

/*---------------------------------------------------------------------------*/

void report(
		Result_t			r,
		const char			*file,
		const char			*function,
		int					line
	)
{
	char c[8192];

	sprintf(c, "%s: %s (%s, %s, %d)\n",
		r.type == RESULT_ERROR ? "Error" : "Warning",
		r.message, file, function, line);

	if (r.type == RESULT_ERROR || (!report_occured || strcmp(c, last_report) != 0)) {
		printf("%s", c);

#ifdef LOG_TO_FILE

		fprintf(logfile, "%s", c);

#endif /* LOG_TO_FILE */

	}

	if (r.type == RESULT_ERROR) {
		ResultRelease();
		exit(1);
	}

	if (last_report)
		free(last_report);

	last_report = duplicateString(c);

	report_occured = 1;

}

/*---------------------------------------------------------------------------*/

void ResultUpdateMessage(
		u32					width
	)
{
	if (sceneGraph.textIsAlive && last_report) {
		char c[8192];

		sprintf(c, "~c7009%s", last_report);
		sceneGraph.text->print(c, -0.95f, 0.8f, width, 50);
	}
}

/*---------------------------------------------------------------------------*/

void glAssert(
		const char			*file,
		const char			*function,
		int					line
	)
{
	GLenum result;
	result = glGetError();

	if (result) {
		char str[4096];

		sprintf(str, "GL Assertion failed in %s, %s on line %d with error code %x", file, function, line, result);

		{
			Result_t msg = {RESULT_WARNING, str};

			REPORT(msg);
		}

		printf("%x\n", result);


	}
}

/*---------------------------------------------------------------------------*/

