/*
 * Defines a Sprite node.
 * Inherits from Geometry.
 */

#ifndef __RC_SPRITE_H__
#define __RC_SPRITE_H__

class Sprite : public Geometry {
	friend class SceneGraph;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		Sprite(
				const char				*nname,
				Texture					*ntexture,

				bool					instantiate_resources
			);

		Sprite(
				const char				*nname,
				const char				*texture_filename,

				u32						purge_level
			);

		~Sprite();

	public:
		void setTexture(
				Texture					*tex,

				bool					instantiate_resource
			);

		void setTexture(
				const char				*filename,

				u32						purge_level
			);

		void setSize(
				vec2f					nsize
			);

		/* Can be used for texture atlases */
		void setTexcoords(
				vec2f					from_uv,
				vec2f					to_uv
			);

	private:
		void setup_sprite(
				const char				*nname,
				Texture					*ntexture,

				bool					instantiate_resources
			);

		void drawSelf(
				FrameInfo_t				*di
			);

		Node *duplicateSelf();


/*------------------------------------------------------------[ Members ]----*/

	public:
		vec2f			size;			/* Width and height of sprite */
		vec2f			texcoordsFrom;
		vec2f			texcoordsTo;

	private:
		VertexArray		*vertexArray;
		IndexArray		*indexArray;
		Texture			*texture;

		ShaderProgram	*shaderProgram;

};


#endif /* __RC_SPRITE_H__ */

