/*
 * Defines a State node.
 * Inherits from Node.
 *
 * This sets the rendering state. The state specified in a State node will
 * hold for the whole subtree of the node, or until a new State node is comes
 * along.
 */

#ifndef __RC_STATE_H__
#define __RC_STATE_H__

class State : public Node {
	friend class SceneGraph;
	friend class World;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		State(
				char	*nname
			);

		~State();

	public:

		/* Enable/disable depth testing */

		void enableDepth(
				StateCompare_t	depth_test
			);

		void disableDepth();


		/* Enable/disable stencil testing */

		void enableStencil(
				StateCompare_t	stencil_test,
				i32				ref,
				u32				mask
			);

		void disableStencil();


		/* Enable/disable blending */

		void enableBlend(
				StateSrcBlend_t	src_blend,
				StateDstBlend_t	dst_blend
			);

		void disableBlend();


		/* Enable/disable triangle culling */

		void enableCulling(
				StateCulling_t	cull_strategy
			);

		void disableCulling();


		/* Enable/disable scissor test */

		void enableScissor(
				i32				x,
				i32				y,
				u32				width,
				u32				height
			);

		void disableScissor();


	private:

		void draw(
				FrameInfo_t			*di
			);

		Node *duplicateSelf();

/*------------------------------------------------------------[ Members ]----*/

	private:
		RenderState			state;

};


#endif /* __RC_STATE_H__ */

