/* TODO: Clean up, ffs! */

#include "RenderChimp.h"
#include "SceneGraph.h"
#include "Text.h"


f32 text_quad_tex[8] =
	{1.0f / 16.0f, 1.0f / 16.0f,
	 1.0f / 16.0f, 0.0f,
	 0.0f, 1.0f / 16.0f,
	 0.0f, 0.0f};

f32 text_quad_pos[8] =
	{1.0f, 0.0f,
	1.0f, 1.0f,
	0.0f, 0.0f,
	0.0f, 1.0f};

vec2f one_pixel;

#define TEXT_GLYPH_SPACING				2


#define TEXT_MAX_WORD_LENGTH			8

Text::~Text()
{
	u32 i;

	for (i = 0; i < MAX_PRINT_JOBS; i++) {

		if (print_queue[i])
			free(print_queue[i]);

	}
}

void Text::resize(
		f32 res_x,
		f32	res_y
	)
{
	resX = res_x;
	resY = res_y;

	text_quad_pos[0] = text_quad_pos[2] = 2.0f * (((f32) (glyph_width)) / (res_x));
	text_quad_pos[3] = text_quad_pos[7] = 2.0f * (((f32) (glyph_height)) / (res_y));

	one_pixel.x = 2.0f / res_x;
	one_pixel.y = 2.0f / res_y;
}

Text::Text(
		char		*source_texture_filename,
		i32			line_spacing
	)
{
	size_t i, x, y;

	u32 glyph_x = 0, glyph_y = 0;
	u32 bytes_per_pixel;

	unsigned char *c;

	unsigned char *pixels;

	lineSpacing = line_spacing;

	resize(320, 480);

	apeGetTexture(
			source_texture_filename,
			&pixels,
			&width,
			&height,
			&bytes_per_pixel
		);

	if ((width & (width - 1)) || (height & (height - 1))) {
		apeDestroyTexture(pixels);
		return;
	}

	glyph_width = width / 16;
	glyph_height = height / 16;

	c = pixels;

	glyph_tex_w = ((float) glyph_width) / ((float) width);
	glyph_tex_h = ((float) glyph_height) / ((float) height);

	resize(320.0f, 480.0f);

	advance_base[0] = 0;

	/* Get advance */
	for (i = 1; i < 256; i++) {
		u32 max = 0;

		glyph_x = i & 0xF;
		glyph_y = i >> 4;

		advance_base[i] = 0;

		for (y = 0; y < glyph_height; y++) {
			c = pixels + (glyph_y * glyph_height + y) * width + (glyph_x * glyph_width);

			for (x = 0; x < glyph_width; x++) {
				if (*c != 0) {
					max = x > max ? x : max;
				}
				c++;
			}
			
		}

		advance_base[i] = (u8) max;
	}

	glGenTextures(1, &tex);
	glBindTexture(GL_TEXTURE_2D, tex);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_ALPHA, width, height, 0, GL_ALPHA, GL_UNSIGNED_BYTE, pixels);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

	apeDestroyTexture(pixels);

	sprog = sceneGraph.createShaderProgram("TextProg", "\n", "../shaders/TextVS.vs", "../shaders/TextFS.fs", NO_PURGE);

	n_print_jobs = 0;

	for (i = 0; i < MAX_PRINT_JOBS; i++) {
		print_queue_len[i] = 0;
		print_queue[i] = 0;
	}
}




i32 Text::printWord(
		Glyph_t *glyph,
		u32 length,
		i32 x,
		i32 y,
		i32 word_width,
		i32 row_width,
		i32 max_lines,
		i32 *advance,
		i32 *offset_y,
		i32 *current_line
	)
{
	u32 i;
	Glyph_t *g = glyph;

	if (word_width > row_width)
		return 1;

	if (*advance + word_width > row_width) {
		*advance = 0;
		*offset_y += lineSpacing + glyph_height;
		(*current_line)++;
	}

	if (*current_line >= max_lines)
		return 1;

	for (i = 0; i < length; i++) {
		drawGlyph((f32) (x + g->x + *advance), (f32) (y - *offset_y), g);
		g++;
	}

	*advance += word_width;

	return 0;
}




typedef enum {
	NormalText = 0,

	AwaitCommand,

	SwitchColorRed,
	SwitchColorGreen,
	SwitchColorBlue,
	SwitchColorAlpha,

	TextInputStateEnd
} TextInputState_t;

typedef enum {
	TextColor = 0,
	EffectColor
} TextColorType_t;

#define TEXT_BOLD			0x1
#define TEXT_ITALIC			0x2


void Text::print(
		const char		*string,
		f32				from_x,
		f32				from_y,
		i32				row_width,
		i32				max_lines
	)
{
	u32 len = strlen(string);

	print_queue_x[n_print_jobs] = from_x;
	print_queue_y[n_print_jobs] = from_y;
	print_queue_rwdt[n_print_jobs] = row_width;
	print_queue_maxl[n_print_jobs] = max_lines;
//	print_queue[n_print_jobs] = duplicateString(string);

	if (!print_queue[n_print_jobs]) {
		print_queue_len[n_print_jobs] = len << 1;
		print_queue[n_print_jobs] = (char *) malloc(print_queue_len[n_print_jobs] + 1);
	} else if (print_queue_len[n_print_jobs] < len) {
		print_queue_len[n_print_jobs] = len << 1;
		print_queue[n_print_jobs] = (char *) realloc(print_queue[n_print_jobs], print_queue_len[n_print_jobs] + 1);
	}

	memcpy(print_queue[n_print_jobs], string, len);
	print_queue[n_print_jobs][len] = '\0';

	n_print_jobs++;
}


void Text::printAll()
{
	u32 i;


	GL_ASSERT();

	sprog->activate();
	glBindTexture(GL_TEXTURE_2D, tex);

	sprog->setAttributePointer("pos", 2, false, 0, text_quad_pos);
	sprog->setAttributePointer("tex", 2, false, 0, text_quad_tex);

	glUniform1i(glGetUniformLocation(sprog->getID(), "smp2d"), 0);

	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


	for (i = 0; i < n_print_jobs; i++) {
		printOne(print_queue[i], print_queue_x[i], print_queue_y[i],
			print_queue_rwdt[i], print_queue_maxl[i]);
	}

	n_print_jobs = 0;
}

void Text::printOne(
		const char		*string,
		f32				from_x,
		f32				from_y,
		i32				row_width,
		i32				max_lines
	)
{
	i32 x, y;
	f32 color[4];
	f32 tcolor[4];
	i32 advance = 0;
	i32 word_advance = 0;
	i32 offset_y;
	i32 current_line = 0;

	TextInputState_t state = NormalText;

	const char *c = string;

	u32 bold = 0;
	u32 italic = 0;

	TextColorType_t switch_text_color = TextColor;

	Glyph_t glyph[TEXT_MAX_WORD_LENGTH];
	u32 glyph_index = 0;

	x = (i32) (from_x * resX * 0.5f);
	y = (i32) (from_y * resY * 0.5f);

	color[0] = 1.0f;
	color[1] = 1.0f;
	color[2] = 1.0f;
	color[3] = 1.0f;

	offset_y = 0;

	for (;;) {

		if (*c == '\0') {
			
			printWord(glyph, glyph_index, x, y, word_advance, row_width, max_lines, &advance, &offset_y, &current_line);
			
			return;

		} else if (state == NormalText) {

			if (*c == ' ' || glyph_index == TEXT_MAX_WORD_LENGTH) {

				if (printWord(glyph, glyph_index, x, y, word_advance, row_width, max_lines, &advance, &offset_y, &current_line))
					return;

				if (glyph_index == TEXT_MAX_WORD_LENGTH)
					c--;

				word_advance = 0;
				glyph_index = 0;

				if (*c == ' ')
					advance += advance_base[32] + TEXT_GLYPH_SPACING;

			} else if (*c == '\n' || *c == '\r') {

				if (printWord(glyph, glyph_index, x, y, word_advance, row_width, max_lines, &advance, &offset_y, &current_line))
					return;

				advance = 0;
				offset_y += lineSpacing + glyph_height;
				current_line++;

				if (current_line >= max_lines)
					return;

				word_advance = 0;
				glyph_index = 0;

			} else if (*c == '~' && c[1] != '~') {

				state = AwaitCommand;

			} else {
				glyph[glyph_index].c = *c;
				glyph[glyph_index].x = word_advance;
				glyph[glyph_index].flags = bold | italic;
				memcpy(glyph[glyph_index].color, color, sizeof(float) * 4);

				glyph_index++;

				word_advance += advance_base[*c] + (bold ? 1 : 0) + TEXT_GLYPH_SPACING;

				if (*c == '~' && c[1] == '~')
					c++;

			}

		} else if (state == AwaitCommand) {

			if (*c == 'c') {

				switch_text_color = TextColor;
				state = SwitchColorRed;

			} else if (*c == 'e') {

				switch_text_color = EffectColor;
				state = SwitchColorRed;

			} else if (*c == 'b') {

				bold ^= TEXT_BOLD;
				state = NormalText;

			} else if (*c == 'i') {

				italic ^= TEXT_ITALIC;
				state = NormalText;

			} else if (*c == 'r') {

				bold = 0;
				italic = 0;
				state = NormalText;

			} else {

				state = NormalText;

			}

		} else if (state >= SwitchColorRed || state <= SwitchColorAlpha) {

			if (*c >= '0' && *c <= '9') {
				tcolor[(int) state - SwitchColorRed] = (float) (*c - '0') / 9.0f;
			} else {
				tcolor[(int) state - SwitchColorRed] = 1.0f;
			}

			(*((i32 *) &state))++;

			if (state > SwitchColorAlpha) {
				if (switch_text_color == TextColor) {
					memcpy(color, tcolor, 4 * sizeof(float));
				}
				state = NormalText;
			}

		}


		c++;
	}


	return;
}

void Text::drawGlyph(
		GLfloat x,
		GLfloat y,
		Glyph_t *g
	)
{
	vec2f offset, /*size,*/ glyph;

	offset.x = 2.0f * (x / (resX));
	offset.y = 2.0f * (y / (resY));

	glyph.x = ((f32) (g->c % 16)) / 16.0f;
	glyph.y = ((f32) (g->c / 16)) / 16.0f;

	glUniform2fv(glGetUniformLocation(sprog->getID(), "offset"), 1, offset.s);
	glUniform2fv(glGetUniformLocation(sprog->getID(), "glyph"), 1, glyph.s);
	glUniform2fv(glGetUniformLocation(sprog->getID(), "pixel_size"), 1, one_pixel.s);
	glUniform4fv(glGetUniformLocation(sprog->getID(), "color"), 1, g->color);

	glUniform1i(glGetUniformLocation(sprog->getID(), "bold"), g->flags & TEXT_BOLD);
	glUniform1i(glGetUniformLocation(sprog->getID(), "italic"), g->flags & TEXT_ITALIC);

	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

}


