/*
 * Texture.
 * Inherits from Resource.
 */

#ifndef __RC_TEXTURE_H__
#define __RC_TEXTURE_H__

/*---------------------------------------------------[ Texture settings ]----*/

typedef enum {

	TEXTURE_FORMAT_RGBA = 0,
	TEXTURE_FORMAT_RGB,
	TEXTURE_FORMAT_GRAY,

	TEXTURE_FORMAT_LAST

} TextureFormat_t;

typedef enum {

	TEXTURE_FILTER_NEAREST = 0,
	TEXTURE_FILTER_BILINEAR,
	TEXTURE_FILTER_TRILINEAR,

	TEXTURE_FILTER_LAST

} TextureFilter_t;

typedef enum {

	TEXTURE_WRAP_CLAMP = 0,
	TEXTURE_WRAP_REPEAT,
	TEXTURE_WRAP_MIRRORED_REPEAT,

	TEXTURE_WRAP_LAST

} TextureWrap_t;

/*---------------------------------------------------------------------------*/



class Texture : public Resource {
	friend class SceneGraph;
	friend class Texture;

/*----------------------------------------------------------[ Functions ]----*/

	private:
		Texture(
				const char			*filename,
				bool				generate_mipmaps,

				TextureFilter_t		filter,
				TextureWrap_t		wrap,

				u32					purge_level
			);

		Texture(
				const u8			*pixels,		/* Color values [0, 255] */
				const char			*nidentifier,

				TextureFormat_t		nformat,

				u32					nwidth,			/* Pixel size */
				u32					nheight,

				bool				generate_mipmaps,

				TextureFilter_t		filter,
				TextureWrap_t		wrap,

				u32					purge_level
			);

		~Texture();

	public:
		Resource *clone(
				const char			*clone_name,

				u32					purge_level
			);

		bool hasMipMaps() { return hasMipmaps; }

		GLuint getGLTexId() { return texId; }

		u32 getWidth() { return width; }

		u32 getHeight() { return height; }

		TextureFormat_t getFormat() { return format; }



	/* Static helper functions */

		static void setTexture(
				Texture				**target_tex,
				Texture				*new_tex,
				bool				instantiate_resource
			);

		static void setTexture(
				Texture				**target_tex,
				const char			*filename,
				u32					purge_level
			);

	private:

		void setup_texture(
				const u8			*pixels,
				TextureFormat_t		nformat,
				u32					nwidth,
				u32					nheight,
				bool				generate_mipmaps,
				TextureFilter_t		filter,
				TextureWrap_t		wrap
			);

#ifdef DEBUG
	protected:
		void print(
				FILE	*f
			);
#endif /* DEBUG */

/*------------------------------------------------------------[ Members ]----*/

	private:
		TextureFormat_t		format;
		u32					width;
		u32					height;

		bool				hasMipmaps;
		GLuint				texId;

};

#endif /* __RC_TEXTURE_H__ */

