#ifdef RC_VMATH


#include "RenderChimp.h"
#include <cmath>

template<class T>
vec2<T>::vec2()
{
}

template<class T>
vec2<T>::vec2(
		const vec2<T>	&v
	)
{
	x = v.x;
	y = v.y;
}

template<class T>
vec2<T>::vec2(
		T			nx,
		T			ny
	)
{
	x = nx;
	y = ny;
}


/* Functions */

/* Assignment */
template<class T>
void vec2<T>::operator=(
		const vec2<T>	&v
	)
{
	x = v.x;
	y = v.y;
}



/* Comparison */
template<class T>
bool vec2<T>::operator==(
		const vec2<T>	&v
	)
{
	return x == v.x &&
		y == v.y;
}

template<class T>
bool vec2<T>::operator!=(
		const vec2<T>	&v
	)
{
	return !(this == v);
}


/* Addition */
template<class T> 
inline vec2<T> vec2<T>::operator+(
		const vec2<T>	&v
	)
{
	return vec2<T>(x + v.x, y + v.y);
}

template<class T> 
inline void vec2<T>::operator+=(
		const vec2<T>	&v
	)
{
	x += v.x;
	y += v.y;
}

template<class T> 
inline vec2<T> vec2<T>::operator+(
		const T			v
	)
{
	return vec2<T>(x + v, y + v);
}

template<class T> 
inline void vec2<T>::operator+=(
		const T			v
	)
{
	x += v;
	y += v;
}

template<class T> 
inline vec2<T> vec2<T>::operator+()
{
	return vec2<T>(x, y);
}


/* Subtract */
template<class T> 
inline vec2<T> vec2<T>::operator-(
		const vec2<T>	&v
	)
{
	return vec2<T>(x - v.x, y - v.y);
}

template<class T> 
inline void vec2<T>::operator-=(
		const vec2<T>	&v
	)
{
	x -= v.x;
	y -= v.y;
}

template<class T> 
inline vec2<T> vec2<T>::operator-(
		const T			v
	)
{
	return vec2<T>(x - v, y - v);
}

template<class T> 
inline void vec2<T>::operator-=(
		const T			v
	)
{
	x -= v;
	y -= v;
}

template<class T> 
inline vec2<T> vec2<T>::operator-()
{
	return vec2<T>(-x, -y);
}


/* Scalar/element-wise multiplication */
template<class T> 
inline vec2<T> vec2<T>::operator*(
		const vec2<T>	&v
	)
{
	return vec2<T>(x * v.x, y * v.y);
}

template<class T> 
inline void vec2<T>::operator*=(
		const vec2<T>	&v
	)
{
	x *= v.x;
	y *= v.y;
}

template<class T> 
inline vec2<T> vec2<T>::operator*(
		const T			v
	)
{
	return vec2<T>(x * v, y * v);
}

template<class T>
inline void vec2<T>::operator*=(
		const T			v
	)
{
	x *= v;
	y *= v;
}


/* Dot product */
template<class T>
inline T vec2<T>::dot(
		const vec2<T>	&v
	)
{
	return x * v.x + y * v.y;
}

template<class T>
inline T vec2<T>::operator^(
		const vec2<T>	&v
	)
{
	return dot(v);
}

/* Division */
template<class T>
inline vec2<T> vec2<T>::operator/(
		const vec2<T>	&v
	)
{
	return vec2<T>(x / v.x, y / v.y);
}

template<class T>
inline void vec2<T>::operator/=(
		const vec2<T>	&v
	)
{
	x /= v.x;
	y /= v.y;
}

template<class T>
inline vec2<T> vec2<T>::operator/(
		const T			v
	)
{
	T t = ((T) 1) / v;
	return vec2<T>(x * t, y * t);
}

template<class T>
inline void vec2<T>::operator/=(
		const T			v
	)
{
	T t = ((T) 1) / v;
	x *= t;
	y *= t;
}


/* Misc useful stuff */
template<class T>
inline T vec2<T>::minElement()
{
	return x < y ? x : y;
}

template<class T>
inline T vec2<T>::maxElement()
{
	return x > y ? x : y;
}

template<class T>
inline void vec2<T>::clamp(
		const T			min,
		const T			max
	)
{
	x = sclamp(x, min, max);
	y = sclamp(y, min, max);
}

template<class T>
inline void vec2<T>::saturate()
{
	x = ssaturate(x);
	y = ssaturate(y);
}

template<class T>
inline void vec2<T>::normalize()
{
	T t = ((T) 1) / length();
	x *= t;
	y *= t;
}

template<class T>
inline T vec2<T>::length()
{
	return sqrt(x * x + y * y);
}

template<class T>
inline T vec2<T>::length2()
{
	return x * x + y * y;
}

template<class T>
inline vec2<T> vec2<T>::reflection(
		const vec2<T>	&v
	)
{
	T nv2 = dot(v) * ((T) 2);
	return nv2 * vec2<T>(x + v.x, y + v.y);
}

template<class T>
inline vec2<T> vec2<T>::refraction(
		const vec2<T>	&v,
		const T			r
	)
{
	T w, k, wk;
	w = -dot(v) * r;
	k = sqrt(((T) 1) + (w - r) * (w + r));
	wk = w - k;
	return vec2<T>(r * x + wk * v.x,
				   r * y + wk * v.y);
}























template<class T>
vec3<T>::vec3()
{
}

template<class T>
vec3<T>::vec3(
		const vec3<T>	&v
	)
{
	x = v.x;
	y = v.y;
	z = v.z;
}

template<class T>
vec3<T>::vec3(
		T			nx,
		T			ny,
		T			nz
	)
{
	x = nx;
	y = ny;
	z = nz;
}


/* Functions */

/* Assignment */
template<class T>
void vec3<T>::operator=(
		const vec3<T>	&v
	)
{
	x = v.x;
	y = v.y;
	z = v.z;
}


/* Comparison */
template<class T>
bool vec3<T>::operator==(
		const vec3<T>	&v
	)
{
	return x == v.x &&
		y == v.y &&
		z == v.z;
}

template<class T>
bool vec3<T>::operator!=(
		const vec3<T>	&v
	)
{
	return !(this == v);
}


/* Addition */
template<class T> 
inline vec3<T> vec3<T>::operator+(
		const vec3<T>	&v
	)
{
	return vec3<T>(x + v.x, y + v.y, z + v.z);
}

template<class T> 
inline void vec3<T>::operator+=(
		const vec3<T>	&v
	)
{
	x += v.x;
	y += v.y;
	z += v.z;
}

template<class T> 
inline vec3<T> vec3<T>::operator+(
		const T			v
	)
{
	return vec3<T>(x + v, y + v, z + v);
}

template<class T> 
inline void vec3<T>::operator+=(
		const T			v
	)
{
	x += v;
	y += v;
	z += v;
}

template<class T> 
inline vec3<T> vec3<T>::operator+()
{
	return vec3<T>(x, y, z);
}


/* Subtract */
template<class T> 
inline vec3<T> vec3<T>::operator-(
		const vec3<T>	&v
	)
{
	return vec3<T>(x - v.x, y - v.y, z - v.z);
}

template<class T> 
inline void vec3<T>::operator-=(
		const vec3<T>	&v
	)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
}

template<class T> 
inline vec3<T> vec3<T>::operator-(
		const T			v
	)
{
	return vec3<T>(x - v, y - v, z - v);
}

template<class T> 
inline void vec3<T>::operator-=(
		const T			v
	)
{
	x -= v;
	y -= v;
	z -= v;
}

template<class T> 
inline vec3<T> vec3<T>::operator-()
{
	return vec3<T>(-x, -y, -z);
}


/* Scalar/element-wise multiplication */
template<class T> 
inline vec3<T> vec3<T>::operator*(
		const vec3<T>	&v
	)
{
	return vec3<T>(x * v.x, y * v.y, z * v.z);
}

template<class T> 
inline void vec3<T>::operator*=(
		const vec3<T>	&v
	)
{
	x *= v.x;
	y *= v.y;
	z *= v.z;
}

template<class T> 
inline vec3<T> vec3<T>::operator*(
		const T			v
	)
{
	return vec3<T>(x * v, y * v, z * v);
}

template<class T>
inline void vec3<T>::operator*=(
		const T			v
	)
{
	x *= v;
	y *= v;
	z *= v;
}


/* Dot product */
template<class T>
inline T vec3<T>::dot(
		const vec3<T>	&v
	)
{
	return x * v.x + y * v.y + z * v.z;
}

template<class T>
inline T vec3<T>::operator^(
		const vec3<T>	&v
	)
{
	return dot(v);
}


/* Cross product */
template<class T>
inline vec3<T> vec3<T>::cross(
		const vec3<T>	&v
	)
{
	return vec3f(y * v.z + z * v.y,
				 z * v.x + x * v.z,
				 x * v.y + y * v.x);
}

template<class T>
vec3<T> vec3<T>::operator%(
		const vec3<T>	&v
	)
{
	return cross(v);
}


/* Division */
template<class T>
inline vec3<T> vec3<T>::operator/(
		const vec3<T>	&v
	)
{
	return vec3<T>(x / v.x, y / v.y, z / v.z);
}

template<class T>
inline void vec3<T>::operator/=(
		const vec3<T>	&v
	)
{
	x /= v.x;
	y /= v.y;
	z /= v.z;
}

template<class T>
inline vec3<T> vec3<T>::operator/(
		const T			v
	)
{
	T t = ((T) 1) / v;
	return vec3<T>(x * t, y * t, z * t);
}

template<class T>
inline void vec3<T>::operator/=(
		const T			v
	)
{
	T t = ((T) 1) / v;
	x *= t;
	y *= t;
	z *= t;
}


/* Misc useful stuff */
template<class T>
inline T vec3<T>::minElement()
{
	return x < y ? (x < z ? x : z) : (y < z ? y : z);
}

template<class T>
inline T vec3<T>::maxElement()
{
	return x > y ? (x > z ? x : z) : (y > z ? y : z);
}

template<class T>
inline void vec3<T>::clamp(
		const T			min,
		const T			max
	)
{
	x = sclamp(x, min, max);
	y = sclamp(y, min, max);
	z = sclamp(z, min, max);
}

template<class T>
inline void vec3<T>::saturate()
{
	x = ssaturate(x);
	y = ssaturate(y);
	z = ssaturate(z);
}

template<class T>
inline void vec3<T>::normalize()
{
	T t = ((T) 1) / length();
	x *= t;
	y *= t;
	z *= t;
}

template<class T>
inline T vec3<T>::length()
{
	return sqrt(x * x + y * y + z * z);
}

template<class T>
inline T vec3<T>::length2()
{
	return x * x + y * y + z * z;
}

template<class T>
inline vec3<T> vec3<T>::reflection(
		const vec3<T>	&v
	)
{
	T nv2 = dot(v) * ((T) 2);
	return nv2 * vec3<T>(x + v.x, y + v.y, z + v.z);
}

template<class T>
inline vec3<T> vec3<T>::refraction(
		const vec3<T>	&v,
		const T			r
	)
{
	T w, k, wk;
	w = -dot(v) * r;
	k = sqrt(((T) 1) + (w - r) * (w + r));
	wk = w - k;
	return vec3<T>(r * x + wk * v.x,
				   r * y + wk * v.y,
				   r * z + wk * v.z);
}























template<class T>
vec4<T>::vec4()
{
}

template<class T>
vec4<T>::vec4(
		const vec4<T>	&v
	)
{
	x = v.x;
	y = v.y;
	z = v.z;
	w = v.w;
}

template<class T>
vec4<T>::vec4(
		T			nx,
		T			ny,
		T			nz,
		T			nw
	)
{
	x = nx;
	y = ny;
	z = nz;
	w = nw;
}


/* Functions */

/* Assignment */
template<class T>
void vec4<T>::operator=(
		const vec4<T>	&v
	)
{
	x = v.x;
	y = v.y;
	z = v.z;
	w = v.w;
}


/* Comparison */
template<class T>
bool vec4<T>::operator==(
		const vec4<T>	&v
	)
{
	return x == v.x &&
		y == v.y &&
		z == v.z &&
		w == v.w;
}

template<class T>
bool vec4<T>::operator!=(
		const vec4<T>	&v
	)
{
	return !(this == v);
}


/* Addition */
template<class T> 
inline vec4<T> vec4<T>::operator+(
		const vec4<T>	&v
	)
{
	return vec4<T>(x + v.x, y + v.y, z + v.z, w + v.w);
}

template<class T> 
inline void vec4<T>::operator+=(
		const vec4<T>	&v
	)
{
	x += v.x;
	y += v.y;
	z += v.z;
}

template<class T> 
inline vec4<T> vec4<T>::operator+(
		const T			v
	)
{
	return vec4<T>(x + v, y + v, z + v, w + v);
}

template<class T> 
inline void vec4<T>::operator+=(
		const T			v
	)
{
	x += v;
	y += v;
	z += v;
	w += v;
}

template<class T> 
inline vec4<T> vec4<T>::operator+()
{
	return vec4<T>(x, y, z, w);
}


/* Subtract */
template<class T> 
inline vec4<T> vec4<T>::operator-(
		const vec4<T>	&v
	)
{
	return vec4<T>(x - v.x, y - v.y, z - v.z, w - v.w);
}

template<class T> 
inline void vec4<T>::operator-=(
		const vec4<T>	&v
	)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;
	w -= v.w;
}

template<class T> 
inline vec4<T> vec4<T>::operator-(
		const T			v
	)
{
	return vec4<T>(x - v, y - v, z - v, w - v);
}

template<class T> 
inline void vec4<T>::operator-=(
		const T			v
	)
{
	x -= v;
	y -= v;
	z -= v;
	w -= v;
}

template<class T> 
inline vec4<T> vec4<T>::operator-()
{
	return vec4<T>(-x, -y, -z, -w);
}


/* Scalar/element-wise multiplication */
template<class T> 
inline vec4<T> vec4<T>::operator*(
		const vec4<T>	&v
	)
{
	return vec4<T>(x * v.x, y * v.y, z * v.z, w * v.w);
}

template<class T> 
inline void vec4<T>::operator*=(
		const vec4<T>	&v
	)
{
	x *= v.x;
	y *= v.y;
	z *= v.z;
	w *= v.w;
}

template<class T> 
inline vec4<T> vec4<T>::operator*(
		const T			v
	)
{
	return vec4<T>(x * v, y * v, z * v, w * v);
}

template<class T>
inline void vec4<T>::operator*=(
		const T			v
	)
{
	x *= v;
	y *= v;
	z *= v;
	w *= v;
}


/* Dot product */
template<class T>
inline T vec4<T>::dot(
		const vec4<T>	&v
	)
{
	return x * v.x + y * v.y + z * v.z + w * v.w;
}

template<class T>
inline T vec4<T>::operator^(
		const vec4<T>	&v
	)
{
	return dot(v);
}


/* Division */
template<class T>
inline vec4<T> vec4<T>::operator/(
		const vec4<T>	&v
	)
{
	return vec4<T>(x / v.x, y / v.y, z / v.z, w / v.w);
}

template<class T>
inline void vec4<T>::operator/=(
		const vec4<T>	&v
	)
{
	x /= v.x;
	y /= v.y;
	z /= v.z;
	w /= v.w;
}

template<class T>
inline vec4<T> vec4<T>::operator/(
		const T			v
	)
{
	T t = ((T) 1) / v;
	return vec4<T>(x * t, y * t, z * t, w * t);
}

template<class T>
inline void vec4<T>::operator/=(
		const T			v
	)
{
	T t = ((T) 1) / v;
	x *= t;
	y *= t;
	z *= t;
	w *= t;
}


/* Misc useful stuff */
template<class T>
inline T vec4<T>::minElement()
{
	T a = x < y ? x : y;
	T b = z < w ? z : w;
	return a < b ? a : b;
}

template<class T>
inline T vec4<T>::maxElement()
{
	T a = x > y ? x : y;
	T b = z > w ? z : w;
	return a > b ? a : b;
}

template<class T>
inline void vec4<T>::clamp(
		const T			min,
		const T			max
	)
{
	x = sclamp(x, min, max);
	y = sclamp(y, min, max);
	z = sclamp(z, min, max);
	w = sclamp(w, min, max);
}

template<class T>
inline void vec4<T>::saturate()
{
	x = ssaturate(x);
	y = ssaturate(y);
	z = ssaturate(z);
	w = ssaturate(w);
}

template<class T>
inline void vec4<T>::normalize()
{
	T f = ((T) 1) / length();
	return vec4<T>(x * f, y * f, z * f, w * f);
}

template<class T>
inline T vec4<T>::length()
{
	return sqrt(x * x + y * y + z * z + w * w);
}

template<class T>
inline T vec4<T>::length2()
{
	return x * x + y * y + z * z + w * w;
}

template<class T>
inline vec4<T> vec4<T>::reflection(
		const vec4<T>	&v
	)
{
	T nv2 = dot(v) * ((T) 2);
	return nv2 * vec4<T>(x + v.x, y + v.y, z + v.z, w + v.w);
}

template<class T>
inline vec4<T> vec4<T>::refraction(
		const vec4<T>	&v,
		const T			r
	)
{
	T w, k, wk;
	w = -dot(v) * r;
	k = sqrt(((T) 1) + (w - r) * (w + r));
	wk = w - k;
	return vec4<T>(r * x + wk * v.x,
				   r * y + wk * v.y,
				   r * z + wk * v.z,
				   r * w + wk * v.w);
}










/* Constructors */
template<class T>
mat2<T>::mat2()
{
}

template<class T>
mat2<T>::mat2(
		const mat2<T>	&v
	)
{
	a = v.a;
	b = v.b;
	c = v.c;
	d = v.d;
}

template<class T>
mat2<T>::mat2(
		T				na,
		T				nb,
		T				nc,
		T				nd
	)
{
	a = na;
	b = nb;
	c = nc;
	d = nd;
}

/* Functions */

/* Assignment */
template<class T>
void mat2<T>::operator=(
		const mat2<T>	&v
	)
{
	a = v.a;
	b = v.b;
	c = v.c;
	d = v.d;
}


/* Comparison */
template<class T>
bool mat2<T>::operator==(
		const mat2<T>	&v
	)
{
	return a == v.a &&
		b == v.b &&
		c == v.c &&
		d == v.d;
}

template<class T>
bool mat2<T>::operator!=(
		const mat2<T>	&v
	)
{
	return !(this == v);
}

template<class T>
bool mat2<T>::isIdentity()
{
	return a == 1 &&
		b == 0 &&
		c == 0 &&
		d == 1;
}


/* Addition */
template<class T>
inline mat2<T> mat2<T>::operator+(
		const mat2<T>	&v
	)
{
	return mat2<T>(
		a + v.a,
		b + v.b,
		c + v.c,
		d + v.d);
}

template<class T>
inline void mat2<T>::operator+=(
		const mat2<T>	&v
	)
{
	a += v.a;
	b += v.b;
	c += v.c;
	d += v.d;
}

template<class T>
inline mat2<T> mat2<T>::operator+(
		const T			v
	)
{
	return mat2<T>(
		a + v,
		b + v,
		c + v,
		d + v);
}

template<class T>
inline void mat2<T>::operator+=(
		const T			v
	)
{
	a += v;
	b += v;
	c += v;
	d += v;
}

template<class T>
inline mat2<T> mat2<T>::operator+()
{
	return mat2<T>(
		a,
		b,
		c,
		d);
}


/* Subtraction */
template<class T>
inline mat2<T> mat2<T>::operator-(
		const mat2<T>	&v
	)
{
	return mat2<T>(
		a - v.a,
		b - v.b,
		c - v.c,
		d - v.d);
}

template<class T>
inline void mat2<T>::operator-=(
		const mat2<T>	&v
	)
{
	a -= v.a;
	b -= v.b;
	c -= v.c;
	d -= v.d;
}

template<class T>
inline mat2<T> mat2<T>::operator-(
		const T			v
	)
{
	return mat2<T>(
		a - v,
		b - v,
		c - v,
		d - v);
}

template<class T>
inline void mat2<T>::operator-=(
		const T			v
	)
{
	a -= v;
	b -= v;
	c -= v;
	d -= v;
}

template<class T>
inline mat2<T> mat2<T>::operator-()
{
	return mat2<T>(
		-a,
		-b,
		-c,
		-d);
}


/* Inner product */
template<class T>
inline T mat2<T>::iprod(
		const mat2<T>	&v,
		const u32		row,
		const u32		col
	)
{
	return t[0][row] * v.t[col][0] +
		t[1][row] * v.t[col][1];
}


/* Multiplication */
template<class T>
mat2<T> mat2<T>::operator*(
		const mat2<T>	&v
	)
{
	return mat2<T>(
			iprod(v, 0, 0),
			iprod(v, 0, 1),
			iprod(v, 1, 0),
			iprod(v, 1, 1));
}

template<class T>
void mat2<T>::operator*=(
		const mat2<T>	&v
	)
{
	a = iprod(v, 0, 0);
	b = iprod(v, 0, 1);
	c = iprod(v, 1, 0);
	d = iprod(v, 1, 1);
}

template<class T>
mat2<T> mat2<T>::operator*(
		const T			v
	)
{
	return mat2<T>(
		a * v,
		b * v,
		c * v,
		d * v);
}

template<class T>
void mat2<T>::operator*=(
		const T			v
	)
{
	a *= v;
	b *= v;
	c *= v;
	d *= v;
}

template<class T>
inline vec2<T> mat2<T>::operator*(
		const vec2<T>	&v
	)
{
	return vec2<T>(
		a * v.x + b * v.y,
		c * v.x + d * v.y
	);
}


/* Element-wise multiplication */
template<class T>
mat2<T> mat2<T>::operator&(
		const mat2<T>	&v
	)
{
	return mat2<T>(
		a * v.a,
		b * v.b,
		c * v.c,
		d * v.d);
}

template<class T>
void mat2<T>::operator&=(
		const mat2<T>	&v
	)
{
	a *= v.a;
	b *= v.b;
	c *= v.c;
	d *= v.d;
}


/* Division with scalar*/
template<class T>
mat2<T> mat2<T>::operator/(
		const T			v
	)
{
	T f = ((T) 1) / v;
	return mat2<T>(
		a * f,
		b * f,
		c * f,
		d * f);
}

template<class T>
void mat2<T>::operator/=(
		const T			v
	)
{
	T f = ((T) 1) / v;
	a *= f;
	b *= f;
	c *= f;
	d *= f;
}


/* Misc matrix operations */
template<class T>
void mat2<T>::identity()
{
	a = 1;
	b = 0;
	c = 0;
	d = 1;
}

template<class T>
void mat2<T>::transpose()
{
	sswap(b, c);
}

template<class T>
T mat2<T>::det()
{
	return a * d - b * c;
}

template<class T>
void mat2<T>::inv()
{
	mat2<T> mat = *this;
	T deter = det();
	T denom;

	if (deter == 0) {
		return mat2<T>(1, 0, 0, 1);
	}

	denom = ((T) 1) / deter();

	a = mat.d * denom;
	b = -mat.b * denom;
	c = -mat.c * denom;
	d = mat.a * denom;
}

template<class T>
T mat2<T>::trace()
{
	return a + d;
}















/* Constructors */
template<class T>
mat3<T>::mat3()
{
}

template<class T>
mat3<T>::mat3(
		const mat3<T>	&v
	)
{
	a = v.a;
	b = v.b;
	c = v.c;

	d = v.d;
	e = v.e;
	f = v.f;

	g = v.g;
	h = v.h;
	i = v.i;
}

template<class T>
mat3<T>::mat3(
		T				na,
		T				nb,
		T				nc,

		T				nd,
		T				ne,
		T				nf,

		T				ng,
		T				nh,
		T				ni
	)
{
	a = na;
	b = nb;
	c = nc;

	d = nd;
	e = ne;
	f = nf;

	g = ng;
	h = nh;
	i = ni;
}

/* Functions */

/* Assignment */
template<class T>
void mat3<T>::operator=(
		const mat3<T>	&v
	)
{
	a = v.a;
	b = v.b;
	c = v.c;

	d = v.d;
	e = v.e;
	f = v.f;

	g = v.g;
	h = v.h;
	i = v.i;
}


/* Comparison */
template<class T>
bool mat3<T>::operator==(
		const mat3<T>	&v
	)
{
	return a == v.a &&
		b == v.b &&
		c == v.c &&

		d == v.d &&
		e == v.e &&
		f == v.f &&

		g == v.g &&
		h == v.h &&
		i == v.i;
}

template<class T>
bool mat3<T>::operator!=(
		const mat3<T>	&v
	)
{
	return !(this == v);
}

template<class T>
bool mat3<T>::isIdentity()
{
	return a == 1 &&
		b == 0 &&
		c == 0 &&

		d == 0 &&
		e == 1 &&
		f == 0 &&

		g == 0 &&
		h == 0 &&
		i == 1;
}


/* Addition */
template<class T>
inline mat3<T> mat3<T>::operator+(
		const mat3<T>	&v
	)
{
	return mat3<T>(
		a + v.a,
		b + v.b,
		c + v.c,

		d + v.d,
		e + v.e,
		f + v.f,

		g + v.g,
		h + v.h,
		i + v.i);
}

template<class T>
inline void mat3<T>::operator+=(
		const mat3<T>	&v
	)
{
	a += v.a;
	b += v.b;
	c += v.c;

	d += v.d;
	e += v.e;
	f += v.f;

	g += v.g;
	h += v.h;
	i += v.i;
}

template<class T>
inline mat3<T> mat3<T>::operator+(
		const T			v
	)
{
	return mat3<T>(
		a + v,
		b + v,
		c + v,

		d + v,
		e + v,
		f + v,

		g + v,
		h + v,
		i + v);
}

template<class T>
inline void mat3<T>::operator+=(
		const T			v
	)
{
	a += v;
	b += v;
	c += v;

	d += v;
	e += v;
	f += v;

	g += v;
	h += v;
	i += v;
}

template<class T>
inline mat3<T> mat3<T>::operator+()
{
	return mat3<T>(
		a, b, c,
		d, e, f,
		g, h, i);
}


/* Subtraction */
template<class T>
inline mat3<T> mat3<T>::operator-(
		const mat3<T>	&v
	)
{
	return mat3<T>(
		a - v.a,
		b - v.b,
		c - v.c,

		d - v.d,
		e - v.e,
		f - v.f,

		g - v.g,
		h - v.h,
		i - v.i);
}

template<class T>
inline void mat3<T>::operator-=(
		const mat3<T>	&v
	)
{
	a -= v.a;
	b -= v.b;
	c -= v.c;

	d -= v.d;
	e -= v.e;
	f -= v.f;

	g -= v.g;
	h -= v.h;
	i -= v.i;
}

template<class T>
inline mat3<T> mat3<T>::operator-(
		const T			v
	)
{
	return mat3<T>(
		a - v,
		b - v,
		c - v,

		d - v,
		e - v,
		f - v,

		g - v,
		h - v,
		i - v);
}

template<class T>
inline void mat3<T>::operator-=(
		const T			v
	)
{
	a -= v;
	b -= v;
	c -= v;

	d -= v;
	e -= v;
	f -= v;

	g -= v;
	h -= v;
	i -= v;
}

template<class T>
inline mat3<T> mat3<T>::operator-()
{
	return mat3<T>(
		-a, -b, -c,
		-d, -e, -f,
		-g, -h, -i);
}


/* Inner product */
template<class T>
inline T mat3<T>::iprod(
		const mat3<T>	&v,
		const u32		row,
		const u32		col
	)
{
	return t[0][row] * v.t[col][0] +
		t[1][row] * v.t[col][1] +
		t[2][row] * v.t[col][2];
}


/* Multiplication */
template<class T>
mat3<T> mat3<T>::operator*(
		const mat3<T>	&v
	)
{
	return mat3<T>(
			iprod(v, 0, 0),
			iprod(v, 0, 1),
			iprod(v, 0, 2),

			iprod(v, 1, 0),
			iprod(v, 1, 1),
			iprod(v, 1, 2),

			iprod(v, 2, 0),
			iprod(v, 2, 1),
			iprod(v, 2, 2));
}

template<class T>
void mat3<T>::operator*=(
		const mat3<T>	&v
	)
{
	a = iprod(v, 0, 0);
	b = iprod(v, 0, 1);
	c = iprod(v, 0, 2);

	d = iprod(v, 1, 0);
	e = iprod(v, 1, 1);
	f = iprod(v, 1, 2);

	g = iprod(v, 2, 0);
	h = iprod(v, 2, 1);
	i = iprod(v, 2, 2);
}

template<class T>
mat3<T> mat3<T>::operator*(
		const T			v
	)
{
	return mat3<T>(
		a * v,
		b * v,
		c * v,

		d * v,
		e * v,
		f * v,

		g * v,
		h * v,
		i * v);
}

template<class T>
void mat3<T>::operator*=(
		const T			v
	)
{
	a *= v;
	b *= v;
	c *= v;

	d *= v;
	e *= v;
	f *= v;

	g *= v;
	h *= v;
	i *= v;
}

template<class T>
inline vec3<T> mat3<T>::operator*(
		const vec3<T>	&v
	)
{
	return vec3<T>(
		a * v.x + b * v.y + c * v.z,
		d * v.x + e * v.y + f * v.z,
		g * v.x + h * v.y + i * v.z
	);
}


/* Element-wise multiplication */
template<class T>
mat3<T> mat3<T>::operator&(
		const mat3<T>	&v
	)
{
	return mat3<T>(
		a * v.a,
		b * v.b,
		c * v.c,

		d * v.d,
		e * v.e,
		f * v.f,

		g * v.g,
		h * v.h,
		i * v.i);
}

template<class T>
void mat3<T>::operator&=(
		const mat3<T>	&v
	)
{
	a *= v.a;
	b *= v.b;
	c *= v.c;

	d *= v.d;
	e *= v.e;
	f *= v.f;

	g *= v.g;
	h *= v.h;
	i *= v.i;
}


/* Division with scalar*/
template<class T>
mat3<T> mat3<T>::operator/(
		const T			v
	)
{
	T f = ((T) 1) / v;
	return mat3<T>(
		a * f,
		b * f,
		c * f,

		d * f,
		e * f,
		f * f,

		g * f,
		h * f,
		i * f);
}

template<class T>
void mat3<T>::operator/=(
		const T			v
	)
{
	T f = ((T) 1) / v;
	a *= f;
	b *= f;
	c *= f;

	d *= f;
	e *= f;
	f *= f;

	g *= f;
	h *= f;
	i *= f;
}


/* Misc matrix operations */
template<class T>
void mat3<T>::identity()
{
	a = e = i = 1;
	b = c = 0;
	d = f = 0;
	g = h = 0;
}

template<class T>
void mat3<T>::transpose()
{
	sswap(b, d);
	sswap(c, g);
	sswap(f, h);
}

template<class T>
T mat3<T>::det()
{
	return a * (i * e - h * f) -
		d * (i * b - h * c) +
		g * (f * b - e * c);
}

template<class T>
void mat3<T>::inv()
{
	mat3<T> mat = *this;
	T deter = det();
	T denom;

	if (deter == 0) {
		identity();
		return;
	}

	denom = ((T) 1) / det();

	a = (mat.e * mat.i - mat.f * mat.h) * denom;
	b = (mat.c * mat.h - mat.b * mat.i) * denom;
	c = (mat.b * mat.f - mat.c * mat.e) * denom;

	d = (mat.f * mat.g - mat.d * mat.i) * denom;
	e = (mat.a * mat.i - mat.c * mat.g) * denom;
	f = (mat.d * mat.c - mat.a * mat.f) * denom;

	g = (mat.d * mat.h - mat.e * mat.g) * denom;
	h = (mat.b * mat.g - mat.a * mat.h) * denom;
	i = (mat.a * mat.e - mat.b * mat.d) * denom;
}


template<class T>
T mat3<T>::trace()
{
	return a + e + i;
}





























/* Constructors */
template<class T>
mat4<T>::mat4()
{
}

template<class T>
mat4<T>::mat4(
		const mat4<T>	&v
	)
{
	a = v.a;
	b = v.b;
	c = v.c;
	d = v.d;

	e = v.e;
	f = v.f;
	g = v.g;
	h = v.h;

	i = v.i;
	j = v.j;
	k = v.k;
	l = v.l;

	m = v.m;
	n = v.n;
	o = v.o;
	p = v.p;
}

template<class T>
mat4<T>::mat4(
		T				na,
		T				nb,
		T				nc,
		T				nd,

		T				ne,
		T				nf,
		T				ng,
		T				nh,

		T				ni,
		T				nj,
		T				nk,
		T				nl,

		T				nm,
		T				nn,
		T				no,
		T				np
	)
{
	a = na;
	b = nb;
	c = nc;
	d = nd;

	e = ne;
	f = nf;
	g = ng;
	h = nh;

	i = ni;
	j = nj;
	k = nk;
	l = nl;

	m = nm;
	n = nn;
	o = no;
	p = np;
}

/* Functions */

/* Assignment */
template<class T>
void mat4<T>::operator=(
		const mat4<T>	&v
	)
{
	a = v.a;
	b = v.b;
	c = v.c;
	d = v.d;

	e = v.e;
	f = v.f;
	g = v.g;
	h = v.h;

	i = v.i;
	j = v.j;
	k = v.k;
	l = v.l;

	m = v.m;
	n = v.n;
	o = v.o;
	p = v.p;
}


/* Comparison */
template<class T>
bool mat4<T>::operator==(
		const mat4<T>	&v
	)
{
	return a == v.a &&
		b == v.b &&
		c == v.c &&
		d == v.d &&

		e == v.e &&
		f == v.f &&
		g == v.g &&
		h == v.h &&

		i == v.i &&
		j == v.j &&
		k == v.k &&
		l == v.l &&

		m == v.m &&
		n == v.n &&
		o == v.o &&
		p == v.p;
}

template<class T>
bool mat4<T>::operator!=(
		const mat4<T>	&v
	)
{
	return !(this == v);
}

template<class T>
bool mat4<T>::isIdentity()
{
	return a == 1 &&
		b == 0 &&
		c == 0 &&
		d == 0 &&

		e == 0 &&
		f == 1 &&
		g == 0 &&
		h == 0 &&

		i == 0 &&
		j == 0 &&
		k == 1 &&
		l == 0 &&

		m == 0 &&
		n == 0 &&
		o == 0 &&
		p == 1;
}


/* Addition */
template<class T>
inline mat4<T> mat4<T>::operator+(
		const mat4<T>	&v
	)
{
	return mat4<T>(
		a + v.a,
		b + v.b,
		c + v.c,
		d + v.d,

		e + v.e,
		f + v.f,
		g + v.g,
		h + v.h,

		i + v.i,
		j + v.j,
		k + v.k,
		l + v.l,

		m + v.m,
		n + v.n,
		o + v.o,
		p + v.p);
}

template<class T>
inline void mat4<T>::operator+=(
		const mat4<T>	&v
	)
{
	a += v.a;
	b += v.b;
	c += v.c;
	d += v.d;

	e += v.e;
	f += v.f;
	g += v.g;
	h += v.h;

	i += v.i;
	j += v.j;
	k += v.k;
	l += v.l;

	m += v.m;
	n += v.n;
	o += v.o;
	p += v.p;
}

template<class T>
inline mat4<T> mat4<T>::operator+(
		const T			v
	)
{
	return mat4<T>(
		a + v,
		b + v,
		c + v,
		d + v,

		e + v,
		f + v,
		g + v,
		h + v,

		i + v,
		j + v,
		k + v,
		l + v,

		m + v,
		n + v,
		o + v,
		p + v);
}

template<class T>
inline void mat4<T>::operator+=(
		const T			v
	)
{
	a += v;
	b += v;
	c += v;
	d += v;

	e += v;
	f += v;
	g += v;
	h += v;

	i += v;
	j += v;
	k += v;
	l += v;

	m += v;
	n += v;
	o += v;
	p += v;
}

template<class T>
inline mat4<T> mat4<T>::operator+()
{
	return mat4<T>(
		a, b, c, d,
		e, f, g, h,
		i, j, k, l,
		m, n, o, p);
}


/* Subtraction */
template<class T>
inline mat4<T> mat4<T>::operator-(
		const mat4<T>	&v
	)
{
	return mat4<T>(
		a - v.a,
		b - v.b,
		c - v.c,
		d - v.d,

		e - v.e,
		f - v.f,
		g - v.g,
		h - v.h,

		i - v.i,
		j - v.j,
		k - v.k,
		l - v.l,

		m - v.m,
		n - v.n,
		o - v.o,
		p - v.p);
}

template<class T>
inline void mat4<T>::operator-=(
		const mat4<T>	&v
	)
{
	a -= v.a;
	b -= v.b;
	c -= v.c;
	d -= v.d;

	e -= v.e;
	f -= v.f;
	g -= v.g;
	h -= v.h;

	i -= v.i;
	j -= v.j;
	k -= v.k;
	l -= v.l;

	m -= v.m;
	n -= v.n;
	o -= v.o;
	p -= v.p;
}

template<class T>
inline mat4<T> mat4<T>::operator-(
		const T			v
	)
{
	return mat4<T>(
		a - v,
		b - v,
		c - v,
		d - v,

		e - v,
		f - v,
		g - v,
		h - v,

		i - v,
		j - v,
		k - v,
		l - v,

		m - v,
		n - v,
		o - v,
		p - v);
}

template<class T>
inline void mat4<T>::operator-=(
		const T			v
	)
{
	a -= v;
	b -= v;
	c -= v;
	d -= v;

	e -= v;
	f -= v;
	g -= v;
	h -= v;

	i -= v;
	j -= v;
	k -= v;
	l -= v;

	m -= v;
	n -= v;
	o -= v;
	p -= v;
}

template<class T>
inline mat4<T> mat4<T>::operator-()
{
	return mat4<T>(
		-a, -b, -c, -d,
		-e, -f, -g, -h,
		-i, -j, -k, -l,
		-m, -n, -o, -p);
}


/* Inner product */
template<class T>
inline T mat4<T>::iprod(
		const mat4<T>	&v,
		const u32		row,
		const u32		col
	)
{
	return t[0][row] * v.t[col][0] +
		t[1][row] * v.t[col][1] +
		t[2][row] * v.t[col][2] +
		t[3][row] * v.t[col][3];
}


/* Multiplication */
template<class T>
inline mat4<T> mat4<T>::operator*(
		const mat4<T>	&v
	)
{
	return mat4<T>(
			iprod(v, 0, 0),
			iprod(v, 0, 1),
			iprod(v, 0, 2),
			iprod(v, 0, 3),

			iprod(v, 1, 0),
			iprod(v, 1, 1),
			iprod(v, 1, 2),
			iprod(v, 1, 3),

			iprod(v, 2, 0),
			iprod(v, 2, 1),
			iprod(v, 2, 2),
			iprod(v, 2, 3),

			iprod(v, 3, 0),
			iprod(v, 3, 1),
			iprod(v, 3, 2),
			iprod(v, 3, 3));
}

template<class T>
inline void mat4<T>::operator*=(
		const mat4<T>	&v
	)
{
	a = iprod(v, 0, 0);
	b = iprod(v, 0, 1);
	c = iprod(v, 0, 2);
	d = iprod(v, 0, 3);

	e = iprod(v, 1, 0);
	f = iprod(v, 1, 1);
	g = iprod(v, 1, 2);
	h = iprod(v, 1, 3);

	i = iprod(v, 2, 0);
	j = iprod(v, 2, 1);
	k = iprod(v, 2, 2);
	l = iprod(v, 2, 3);

	m = iprod(v, 3, 0);
	n = iprod(v, 3, 1);
	o = iprod(v, 3, 2);
	p = iprod(v, 3, 3);
}

template<class T>
inline mat4<T> mat4<T>::operator*(
		const T			v
	)
{
	return mat4<T>(
		a * v,
		b * v,
		c * v,
		d * v,

		e * v,
		f * v,
		g * v,
		h * v,

		i * v,
		j * v,
		k * v,
		l * v,

		m * v,
		n * v,
		o * v,
		p * v);
}

template<class T>
inline void mat4<T>::operator*=(
		const T			v
	)
{
	a *= v;
	b *= v;
	c *= v;
	d *= v;

	e *= v;
	f *= v;
	g *= v;
	h *= v;

	i *= v;
	j *= v;
	k *= v;
	l *= v;

	m *= v;
	n *= v;
	o *= v;
	p *= v;
}

template<class T>
inline vec4<T> mat4<T>::operator*(
		const vec4<T>	&v
	)
{
	return vec4<T>(
		a * v.x + b * v.y + c * v.z + d * v.w,
		e * v.x + f * v.y + g * v.z + h * v.w,
		i * v.x + j * v.y + k * v.z + l * v.w,
		m * v.x + n * v.y + o * v.z + p * v.w
	);
}


/* Element-wise multiplication */
template<class T>
inline mat4<T> mat4<T>::operator&(
		const mat4<T>	&v
	)
{
	return mat4<T>(
		a * v.a,
		b * v.b,
		c * v.c,
		d * v.d,

		e * v.e,
		f * v.f,
		g * v.g,
		h * v.h,

		i * v.i,
		j * v.j,
		k * v.k,
		l * v.l,

		m * v.m,
		n * v.n,
		o * v.o,
		p * v.p);
}

template<class T>
inline void mat4<T>::operator&=(
		const mat4<T>	&v
	)
{
	a *= v.a;
	b *= v.b;
	c *= v.c;
	d *= v.d;

	e *= v.e;
	f *= v.f;
	g *= v.g;
	h *= v.h;

	i *= v.i;
	j *= v.j;
	k *= v.k;
	l *= v.l;

	m *= v.m;
	n *= v.n;
	o *= v.o;
	p *= v.p;
}


/* Division with scalar*/
template<class T>
inline mat4<T> mat4<T>::operator/(
		const T			v
	)
{
	T t = ((T) 1) / v;
	return mat4<T>(
		a * t,
		b * t,
		c * t,
		d * t,

		e * t,
		f * t,
		g * t,
		h * t,

		i * t,
		j * t,
		k * t,
		l * t,

		m * t,
		n * t,
		o * t,
		p * t);
}

template<class T>
inline void mat4<T>::operator/=(
		const T			v
	)
{
	T t = ((T) 1) / v;
	a *= t;
	b *= t;
	c *= t;
	d *= t;

	e *= t;
	f *= t;
	g *= t;
	h *= t;

	i *= t;
	j *= t;
	k *= t;
	l *= t;

	m *= t;
	n *= t;
	o *= t;
	p *= t;
}


/* Misc matrix operations */
template<class T>
inline void mat4<T>::identity()
{
	a = f = k = p = 1;
	b = c = d = 0;
	e = g = h = 0;
	i = j = l = 0;
	m = n = o = 0;
}

template<class T>
inline void mat4<T>::transpose()
{
	sswap(e, b);
	sswap(i, c);
	sswap(m, d);
	sswap(j, g);
	sswap(n, h);
	sswap(o, l);
}

template<class T>
inline T mat4<T>::det2(
		T n0,
		T n1,
		T n2,
		T n3
	)
{
	return n0 * n3 - n1 * n2;
}

template<class T>
inline T mat4<T>::det()
{
	T s0, s1, s2, s3, s4, s5;
	T c0, c1, c2, c3, c4, c5;

	s0 = det2(a, b, e, f);
	s1 = det2(a, c, e, g);
	s2 = det2(a, d, e, h);
	s3 = det2(b, c, f, g);
	s4 = det2(b, d, f, h);
	s5 = det2(c, d, g, h);

	c5 = det2(k, l, o, p);
	c4 = det2(j, l, n, p);
	c3 = det2(j, k, n, o);
	c2 = det2(i, l, m, p);
	c1 = det2(i, k, m, o);
	c0 = det2(i, j, m, n);

	return s0 * c5 - s1 * c4 + s2 * c3 + s3 * c2 - s4 * c1 + s5 * c0;
}

template<class T>
inline void mat4<T>::inv()
{
	T s0, s1, s2, s3, s4, s5;
	T c0, c1, c2, c3, c4, c5;
	T deter, denom;

	s0 = det2(a, b, e, f);
	s1 = det2(a, c, e, g);
	s2 = det2(a, d, e, h);
	s3 = det2(b, c, f, g);
	s4 = det2(b, d, f, h);
	s5 = det2(c, d, g, h);

	c5 = det2(k, l, o, p);
	c4 = det2(j, l, n, p);
	c3 = det2(j, k, n, o);
	c2 = det2(i, l, m, p);
	c1 = det2(i, k, m, o);
	c0 = det2(i, j, m, n);

	deter = s0 * c5 - s1 * c4 + s2 * c3 + s3 * c2 - s4 * c1 + s5 * c0;

	if (deter == 0) {
		identity();
		return;
	}

	denom = ((T) 1) / deter;

	mat4<T> mat = *this;

	a = (mat.f * c5 - mat.g * c4 + mat.h * c3) * denom;
	b = (-mat.b * c5 + mat.c * c4 - mat.d * c3) * denom;
	c = (mat.n * s5 - mat.o * s4 + mat.p * s3) * denom;
	d = (-mat.j * s5 + mat.k * s4 - mat.l * s3) * denom;

	e = (-mat.e * c5 + mat.g * c2 - mat.h * c1) * denom;
	f = (mat.a * c5 - mat.c * c2 + mat.d * c1) * denom;
	g = (-mat.m * s5 + mat.o * s2 - mat.p * s1) * denom;
	h = (mat.i * s5 - mat.k * s2 + mat.l * s1) * denom;

	i = (mat.e * c4 - mat.f * c2 + mat.h * c0) * denom;
	j = (-mat.a * c4 + mat.b * c2 - mat.d * c0) * denom;
	k = (mat.m * s4 - mat.n * s2 + mat.p * s0) * denom;
	l = (-mat.i * s4 + mat.j * s2 - mat.l * s0) * denom;

	m = (-mat.e * c3 + mat.f * c1 - mat.g * c0) * denom;
	n = (mat.a * c3 - mat.b * c1 + mat.c * c0) * denom;
	o = (-mat.m * s3 + mat.n * s1 - mat.o * s0) * denom;
	p = (mat.i * s3 - mat.j * s1 + mat.k * s0) * denom;

}

template<class T>
inline T mat4<T>::trace()
{
	return a + f + k + p;
}


/* Matlab-friendly print function */
template<class T>
void mat4<T>::print()
{
	char buf[4096];

	sprintf(buf, "[ %f %f %f %f ;\n %f %f %f %f ;\n %f %f %f %f ;\n %f %f %f %f ]\n",
		a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p);

	printf("%s\n", buf);
}

#endif /* RC_VMATH */
