/*
 * APE - An iPhone Emulator
 *
 */

#ifndef __APE_H__
#define __APE_H__


#define APE_TOUCH_NONE		0
#define APE_TOUCH_SINGLE	1
//#define APE_TOUCH_MULTI		2

/*
 *        ___
 *       / i \
 *       | P |
 *	     | o |   - Pitch [-0.5, 0.5]
 *       | d |
 *       \---/
 *
 *         |
 *
 *  Roll [-0.5, 0.5]
 *
 *
 *  Facing [-0.5, 0.5] :
 *
 *    -0.5 = facing up
 *     0.0 = on side
 *     0.5 = facing down
 */

typedef struct {

	/* Touch */
	u32				touch;
	f32				x;
	f32				y;

	/* Orientation */
	/* TODO: Not supported! */
	f32				roll;
	f32				pitch;
	f32				facing;

	/* Time */
	f32				time;
	f32				timeStep;

} DeviceState_t;




/* This is called at the beginning of the execution */
void RCInit();

/* This is called every frame */
u32 RCUpdate(
		DeviceState_t *dstate
	);

/* This is called when the application is exited */
void RCDestroy();



/* Display functions */

void apeSetViewport(
		i32					from_x,
		i32					to_x,
		u32					from_y,
		u32					to_y
	);

void apeGetDisplaySize(
		u32					*width,
		u32					*height
	);



/* Texture functions */

void apeGetTexture(
		const char			*filename,
		u8					**pixels,
		u32					*width,
		u32					*height,
		u32					*bytes_per_pixel
	);

void apeDestroyTexture(
		unsigned char		*pixels
	);

/* Audio */

// TODO Implement


/* File reading */

#ifdef WIN32

#else if defined MAC
	/* TODO: Define new fopen */
#endif

#endif /* __APE_H__ */


