attribute vec3 Vertex;
attribute vec3 Normal;
attribute vec3 Texcoord;

uniform vec3 ViewPosition;
uniform mat4 World;
uniform mat4 WorldViewProjection;
uniform mat4 WorldInverseTranspose;

#ifdef DIRECTIONAL
	uniform vec3 LightVector;
#else /* POINT */
	uniform vec3 LightPosition;
#endif

uniform vec3 LightColor;

uniform vec3 AmbientColor;
uniform vec3 DiffuseColor;
uniform float Emissive;
uniform vec3 SpecularColor;
uniform float Shininess;
uniform float SpecularLevel;

uniform sampler2D DiffuseSampler;



	// To fragment shader

varying mediump	vec2 texcoord;
varying mediump	vec3 worldPosition;
varying mediump	vec3 worldNormal;
varying mediump	vec3 viewVector;

varying mediump vec3 diffuse_color;



void main() {

		// Setup
	worldPosition = (World * vec4(Vertex.xyz, 1.0)).xyz;
	worldNormal = (WorldInverseTranspose * vec4(Normal.xyz, 1.0)).xyz;
	viewVector = normalize(ViewPosition - worldPosition);
	texcoord = vec2(Texcoord.x, Texcoord.y);

		// Diffuse
		
#ifdef DIRECTIONAL
	vec3 lightVector = LightVector;
#else /* POINT */
	vec3 lightVector = normalize(LightPosition - worldPosition);
#endif

	float lightNormalCos = max(0.0, dot(normalize(worldNormal), lightVector));
	diffuse_color = LightColor * lightNormalCos * DiffuseColor;
	diffuse_color += AmbientColor + vec3(Emissive);


		// Position
	gl_Position = WorldViewProjection * vec4(Vertex.xyz, 1.0);

}

