#ifdef GL_FRAGMENT_PRECISION_HIGH
   precision highp float;
#else
   precision mediump float;
#endif

uniform vec3 LightPosition;

uniform vec3 LightColor;

uniform	vec3 AmbientColor;
uniform	vec3 DiffuseColor;
uniform	vec3 SpecularColor;
uniform	float Shininess;

uniform sampler2D DiffuseSampler;
uniform sampler2D NoiseSampler;

varying	vec2 texcoord;
varying	vec3 worldPosition;
varying	vec3 worldNormal;
varying	vec3 viewVector;

varying vec3 diffuse_color;

uniform float Time;


vec4 specularPhong(
		float specular_level
	)
{
	vec3 specular = vec3(0.0);

	if (specular_level > 0.0) {
		vec3 normWorldNormal = normalize(worldNormal);

		vec3 lightVector = normalize(LightPosition - worldPosition);

		vec3 reflected = -reflect(lightVector, normWorldNormal);
		float s = max(0.0, dot(normalize(reflected), viewVector));
		specular = LightColor * pow(s, Shininess) * specular_level * SpecularColor;
	}

	return vec4(specular, 1.0);
}

void main() {
	vec4 color = vec4(diffuse_color, 1.0);
	
	vec4 noise = texture2D(NoiseSampler, texcoord);
	
	float n = abs(1.0 - mod(noise.x + Time, 1.0) * 2.0);

	color += vec4(vec3(mod(n * 1.25, 1.0)) * DiffuseColor, 1.0);
	
	    // Diffuse texture
	color *= texture2D(DiffuseSampler, texcoord);
	
		// Specular
	color += specularPhong(noise.x * 2.5);
	
		// Write final color to color buffer
	gl_FragColor = color;

}

